package com.example.paybyworkout;

import com.example.paybyworkout.UI.fragments.LoginFragment;
import com.example.paybyworkout.UI.fragments.RegisterFragment;
import com.example.paybyworkout.security.SecurePreferences;
import com.example.paybyworkout.server.Server;

import android.app.Activity;
import android.app.ProgressDialog;
import android.os.Bundle;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.CheckBox;
import android.widget.EditText;
import android.widget.Toast;

/**
 * Gets executed at the starting of the application and is responsible for handling the login and registration process
 * @author Jan
 */
public class LoginActivity extends Activity {

	private Server server;
	private ProgressDialog progressDialog;
	private SecurePreferences preferences;	

	protected void onCreate(Bundle savedInstanceState) {
		
		server = new Server(LoginActivity.this);
		
		progressDialog = new ProgressDialog(this, R.style.MyTheme);
		progressDialog.setCancelable(false);
		
		super.onCreate(savedInstanceState);
		setContentView(R.layout.login_main);
		if (savedInstanceState == null) {
			getFragmentManager().beginTransaction().replace(R.id.container, new LoginFragment()).commit();
		}

		//Get credentials from DB
		preferences = new SecurePreferences(this, "UserCredentials", "eWi2AvSjwR93rgf349edGeo2ru0ajsFfWvjgfG", true);
		
		//No automatic login when coming from logout
		boolean automaticLogin = getIntent().getBooleanExtra("automaticLogin", true);
		
		if(automaticLogin) {
			String username = preferences.getString("username");
			String password = preferences.getString("password");
			
			if(username != null || password != null) {
				server.checkLogin(username, password);
			}
		}
		
		
		//server.checkLogin("xy", "xy");
	}


	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		//getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		// Handle action bar item clicks here. The action bar will
		// automatically handle clicks on the Home/Up button, so long
		// as you specify a parent activity in AndroidManifest.xml.
		return super.onOptionsItemSelected(item);
	}	
	
	/**
	 * Executed by clicking on the login button. Reads out credentials and sends them to the server.
	 * @param view
	 */
	public void login(View view) {
		
		//Checking credentials
		String username = ((EditText) findViewById(R.id.loginUsername)).getText().toString();
		String password = ((EditText) findViewById(R.id.loginPassword)).getText().toString();
		
		if(((CheckBox)findViewById(R.id.automaticLogin)).isChecked()) {
			preferences.put("username", username);
			preferences.put("password", password);		
		}else{
			preferences.put("username", null);
			preferences.put("password", null);
		}		
		
		server.checkLogin(username, password);
	}
	
	/**
	 * Opens the register fragment for unregistered users after clicking on the registration link at the login fragment.
	 * @param view
	 */
	public void register(View view) {
		getFragmentManager().beginTransaction().replace(R.id.container, new RegisterFragment()).commit();		
	}
	
	/**
	 * Executed by clicking the register button. Reads out register data and sends them to server.
	 * @param view
	 */
	public void registerDo(View view) {
		//Insert data
		String username = ((EditText) findViewById(R.id.registerUsername)).getText().toString();
		String password = ((EditText) findViewById(R.id.registerPassword)).getText().toString();
		String bodyWeight = ((EditText) findViewById(R.id.weight)).getText().toString();
		
		if(username.equals("")) {
			Toast.makeText(getApplicationContext(), R.string.username_empty, Toast.LENGTH_LONG).show();							
		}else if(password.equals("")) {
			Toast.makeText(getApplicationContext(), R.string.password_empty, Toast.LENGTH_LONG).show();								
		}else if(bodyWeight.equals("")) {
			Toast.makeText(getApplicationContext(), R.string.no_bodyweight_selected, Toast.LENGTH_LONG).show();								
		}else{
			server.registerUser(username, password, bodyWeight);
		}
	}
	
	/**
	 * Changes the main fragment from register back to login.
	 * @param view
	 */
	public void registerBack(View view) {
		getFragmentManager().beginTransaction().replace(R.id.container, new LoginFragment()).commit();		
	}
	
	/**
	 * Shows the "in progress" icon.
	 */
	public void showProgress() {
		progressDialog.show();
	}
	
	/**
	 * Hides the "in progress" icon.
	 */	
	public void hideProgress() {
		progressDialog.dismiss();
	}
}
