package com.example.paybyworkout.UI.adapters;

import com.example.paybyworkout.R;
import com.example.paybyworkout.classes.Statistic;
import com.example.paybyworkout.classes.WorkoutMode;

import android.app.Activity;
import android.content.Context;
import android.view.LayoutInflater;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ArrayAdapter;
import android.widget.ImageView;
import android.widget.TextView;

/**
 * Adapter, responsible for the UI of the statistic fragment. Gets an array of all statistics and creates one row per item.
 * @author Jan
 */
public class StatisticAdapter extends ArrayAdapter<Statistic> {
	
	   Context context;
	    int layoutResourceId;   
	    Statistic data[] = null;
	   
	    public StatisticAdapter(Context context, int layoutResourceId, Statistic[] data) {
	        super(context, layoutResourceId, data);
	        this.layoutResourceId = layoutResourceId;
	        this.context = context;
	        this.data = data;
	    }

	    @Override
	    public View getView(int position, View convertView, ViewGroup parent) {
	        View row = convertView;
	        StatisticHolder holder = null;
	       
	        if(row == null)
	        {
	            LayoutInflater inflater = ((Activity)context).getLayoutInflater();
	            row = inflater.inflate(layoutResourceId, parent, false);
	           
	            holder = new StatisticHolder();
	            holder.time = (TextView)row.findViewById(R.id.statisticsItemTime);
	            holder.credits = (TextView)row.findViewById(R.id.statisticsItemCredits);
	            holder.image = (ImageView)row.findViewById(R.id.statisticsItemImage);
	           
	            row.setTag(holder);
	        }
	        else
	        {
	            holder = (StatisticHolder)row.getTag();
	        }
	       
	        Statistic statistic = data[position];
	        holder.time.setText(statistic.getStartTimeAsDate());
	        holder.credits.setText(statistic.getCredits() + " " + row.getResources().getString(R.string.credits_earned));
	        if(statistic.getWorkoutMode().equals(WorkoutMode.RUNNING)) holder.image.setImageResource(R.drawable.statistic_running);
	        if(statistic.getWorkoutMode().equals(WorkoutMode.CYCLING)) holder.image.setImageResource(R.drawable.statistic_cycling);
	       
	        return row;
	    }
	   
	    static class StatisticHolder
	    {
	        TextView time;
	        TextView credits;
	        ImageView image;
	    }
}
