package com.example.paybyworkout;

import java.util.ArrayList;

import com.example.paybyworkout.UI.fragments.SettingsFragment;
import com.example.paybyworkout.UI.fragments.StatisticsDetailFragment;
import com.example.paybyworkout.UI.fragments.StatisticsFragment;
import com.example.paybyworkout.UI.fragments.WorkoutFragment;
import com.example.paybyworkout.classes.WorkoutMode;
import com.example.paybyworkout.classes.Statistic;
import com.example.paybyworkout.classes.UserProfile;
import com.example.paybyworkout.database.ExportDatabaseCSVTask;
import com.example.paybyworkout.database.MySQLiteHelper;
import com.example.paybyworkout.featureextraction.FeatureExtractionThread;
import com.example.paybyworkout.record.RecordSensorsThread;
import com.example.paybyworkout.server.Server;

import android.support.v7.app.ActionBarActivity;
import android.annotation.SuppressLint;
import android.app.AlertDialog;
import android.app.Fragment;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.Bundle;
import android.util.Log;
import android.view.Menu;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.AdapterView.OnItemClickListener;
import android.widget.EditText;
import android.widget.ImageView;
import android.widget.LinearLayout;
import android.widget.TextView;

/**
 * Contains all application functions after login (Recording workouts, show statistics, change settings and logout)
 * @author Jan
 *
 */
@SuppressLint("NewApi")
public class WorkoutActivity extends ActionBarActivity implements OnItemClickListener {

	private Intent intent;
	private UserProfile user;
	private ArrayList<Statistic> statistics;
	private Server server;
	private Fragment workoutFragment;
	private Fragment statisticsFragment;
	private RecordSensorsThread recordSensorsThread;
	private FeatureExtractionThread featureExtractionThread;
	private ProgressDialog progressDialog;
	private MySQLiteHelper sqlHelper;
	private SQLiteDatabase database;
	private boolean recording = false;
	private long startRecordingTime = 0;
	private WorkoutMode workoutMode = WorkoutMode.RUNNING;

	@SuppressWarnings("unchecked")
	@Override
	protected void onCreate(Bundle savedInstanceState) {

		server = new Server(WorkoutActivity.this);

		super.onCreate(savedInstanceState);
		setContentView(R.layout.workout_main);	
		if (savedInstanceState == null) {
			workoutFragment = new WorkoutFragment();
			getFragmentManager().beginTransaction().replace(R.id.container, workoutFragment).commit();
		}

		progressDialog = new ProgressDialog(this, R.style.MyTheme);
		progressDialog.setCancelable(false);

		sqlHelper = new MySQLiteHelper(this);
		database = sqlHelper.getWritableDatabase();

		user = (UserProfile) getIntent().getSerializableExtra("user");
		statistics = (ArrayList<Statistic>) getIntent().getSerializableExtra("statistics");        	
	}

	@Override
	public boolean onCreateOptionsMenu(Menu menu) {
		// Inflate the menu; this adds items to the action bar if it is present.
		//getMenuInflater().inflate(R.menu.main, menu);
		return true;
	}

	/**
	 * Shows the fragment for recording workouts
	 * @param view
	 */
	@SuppressWarnings("deprecation")
	public void menuWorkout(View view) {
		if(!recording) {
			getFragmentManager().beginTransaction().replace(R.id.container, workoutFragment).commit();

			ImageView image = (ImageView) findViewById(R.id.menuWorkoutImage);
			image.setImageResource(R.drawable.workout);
			TextView text = (TextView) findViewById(R.id.menuWorkoutText);
			text.setTextColor(getResources().getColor(R.color.menuTextActive));
			image = (ImageView) findViewById(R.id.menuStatisticsImage);
			image.setImageResource(R.drawable.statistics_inactive);
			text = (TextView) findViewById(R.id.menuStatisticsText);
			text.setTextColor(getResources().getColor(R.color.menuTextInactive));
			image = (ImageView) findViewById(R.id.menuSettingsImage);
			image.setImageResource(R.drawable.settings_inactive);
			text = (TextView) findViewById(R.id.menuSettingsText);
			text.setTextColor(getResources().getColor(R.color.menuTextInactive));

			//Change background
			int sdk = android.os.Build.VERSION.SDK_INT;
			if(sdk < android.os.Build.VERSION_CODES.JELLY_BEAN) {
				LinearLayout layout = (LinearLayout) findViewById(R.id.menuWorkout);
				layout.setBackgroundDrawable(getResources().getDrawable(R.drawable.menu_bg_active));
				layout = (LinearLayout) findViewById(R.id.menuStatistics);
				layout.setBackgroundDrawable(getResources().getDrawable(R.drawable.menu_bg_inactive));
				layout = (LinearLayout) findViewById(R.id.menuSettings);
				layout.setBackgroundDrawable(getResources().getDrawable(R.drawable.menu_bg_inactive));
			} else {
				LinearLayout layout = (LinearLayout) findViewById(R.id.menuWorkout);
				layout.setBackground(getResources().getDrawable(R.drawable.menu_bg_active));
				layout = (LinearLayout) findViewById(R.id.menuStatistics);
				layout.setBackground(getResources().getDrawable(R.drawable.menu_bg_inactive));
				layout = (LinearLayout) findViewById(R.id.menuSettings);
				layout.setBackground(getResources().getDrawable(R.drawable.menu_bg_inactive));
			}
		}
	}

	/**
	 * Shows the fragment for statistics about recent workout
	 * @param view
	 */	@SuppressWarnings("deprecation")
	public void menuStatistics(View view) {
		if(!recording) {
			statisticsFragment = new StatisticsFragment(this, statistics);
			getFragmentManager().beginTransaction().replace(R.id.container, statisticsFragment).commit();

			ImageView image = (ImageView) findViewById(R.id.menuStatisticsImage);
			image.setImageResource(R.drawable.statistics);
			TextView text = (TextView) findViewById(R.id.menuStatisticsText);
			text.setTextColor(getResources().getColor(R.color.menuTextActive));		
			image = (ImageView) findViewById(R.id.menuWorkoutImage);
			image.setImageResource(R.drawable.workout_inactive);
			text = (TextView) findViewById(R.id.menuWorkoutText);
			text.setTextColor(getResources().getColor(R.color.menuTextInactive));		
			image = (ImageView) findViewById(R.id.menuSettingsImage);
			image.setImageResource(R.drawable.settings_inactive);
			text = (TextView) findViewById(R.id.menuSettingsText);
			text.setTextColor(getResources().getColor(R.color.menuTextInactive));	

			//Change background
			int sdk = android.os.Build.VERSION.SDK_INT;
			if(sdk < android.os.Build.VERSION_CODES.JELLY_BEAN) {
				LinearLayout layout = (LinearLayout) findViewById(R.id.menuStatistics);
				layout.setBackgroundDrawable(getResources().getDrawable(R.drawable.menu_bg_active));
				layout = (LinearLayout) findViewById(R.id.menuWorkout);
				layout.setBackgroundDrawable(getResources().getDrawable(R.drawable.menu_bg_inactive));
				layout = (LinearLayout) findViewById(R.id.menuSettings);
				layout.setBackgroundDrawable(getResources().getDrawable(R.drawable.menu_bg_inactive));
			} else {
				LinearLayout layout = (LinearLayout) findViewById(R.id.menuStatistics);
				layout.setBackground(getResources().getDrawable(R.drawable.menu_bg_active));
				layout = (LinearLayout) findViewById(R.id.menuWorkout);
				layout.setBackground(getResources().getDrawable(R.drawable.menu_bg_inactive));
				layout = (LinearLayout) findViewById(R.id.menuSettings);
				layout.setBackground(getResources().getDrawable(R.drawable.menu_bg_inactive));
			}
		}
	}

	/**
	 * Shows the fragment for user settings
	 * @param view
	 */
	 @SuppressWarnings("deprecation")
	public void menuSettings(View view) {
		if(!recording) {
			getFragmentManager().beginTransaction().replace(R.id.container, new SettingsFragment(user)).commit();

			ImageView image = (ImageView) findViewById(R.id.menuSettingsImage);
			image.setImageResource(R.drawable.settings);
			TextView text = (TextView) findViewById(R.id.menuSettingsText);
			text.setTextColor(getResources().getColor(R.color.menuTextActive));		
			image = (ImageView) findViewById(R.id.menuWorkoutImage);
			image.setImageResource(R.drawable.workout_inactive);
			text = (TextView) findViewById(R.id.menuWorkoutText);
			text.setTextColor(getResources().getColor(R.color.menuTextInactive));
			image = (ImageView) findViewById(R.id.menuStatisticsImage);
			image.setImageResource(R.drawable.statistics_inactive);
			text = (TextView) findViewById(R.id.menuStatisticsText);
			text.setTextColor(getResources().getColor(R.color.menuTextInactive));	

			//Change background
			int sdk = android.os.Build.VERSION.SDK_INT;
			if(sdk < android.os.Build.VERSION_CODES.JELLY_BEAN) {
				LinearLayout layout = (LinearLayout) findViewById(R.id.menuSettings);
				layout.setBackgroundDrawable(getResources().getDrawable(R.drawable.menu_bg_active));
				layout = (LinearLayout) findViewById(R.id.menuWorkout);
				layout.setBackgroundDrawable(getResources().getDrawable(R.drawable.menu_bg_inactive));
				layout = (LinearLayout) findViewById(R.id.menuStatistics);
				layout.setBackgroundDrawable(getResources().getDrawable(R.drawable.menu_bg_inactive));
			} else {
				LinearLayout layout = (LinearLayout) findViewById(R.id.menuSettings);
				layout.setBackground(getResources().getDrawable(R.drawable.menu_bg_active));
				layout = (LinearLayout) findViewById(R.id.menuWorkout);
				layout.setBackground(getResources().getDrawable(R.drawable.menu_bg_inactive));
				layout = (LinearLayout) findViewById(R.id.menuStatistics);
				layout.setBackground(getResources().getDrawable(R.drawable.menu_bg_inactive));
			}	
		}
	}

	/**
	 * Shows alert dialog for logging out
	 * @param view
	 */
	 public void menuLogout(View view) {
		if(!recording) {
			new AlertDialog.Builder(this)
			.setIcon(android.R.drawable.ic_dialog_alert)
			.setTitle(R.string.logout_really)
			.setMessage(R.string.all_data_lost)
			.setPositiveButton(R.string.logout, new DialogInterface.OnClickListener()
			{
				@Override
				public void onClick(DialogInterface dialog, int which) {
					intent = new Intent(WorkoutActivity.this, LoginActivity.class);
					intent.putExtra("automaticLogin", false);
					startActivity(intent);
					finish();				
				}

			})
			.setNegativeButton(R.string.cancel, null)
			.show();
		}
	}

	@Override
	public boolean onOptionsItemSelected(MenuItem item) {
		return false;
	}

	/**
	 * Starts recording sensors after clicking on the recording button in the workout fragment and extracts features out of this stream, which get classified within a specified interval.
	 * @param view
	 */
	public void startRecording(View view) {
		findViewById(R.id.startButton).setVisibility(View.GONE);
		findViewById(R.id.stopButton).setVisibility(View.VISIBLE);

		findViewById(R.id.workoutMode).setVisibility(View.GONE);
		findViewById(R.id.status).setVisibility(View.VISIBLE);
		findViewById(R.id.time).setVisibility(View.VISIBLE);
		
		recording = true;
		startRecordingTime = System.currentTimeMillis();

		recordSensorsThread = new RecordSensorsThread(WorkoutActivity.this);
		recordSensorsThread.start();

		featureExtractionThread = new FeatureExtractionThread(WorkoutActivity.this);
		featureExtractionThread.start();

		((TextView)findViewById(R.id.time)).setTextSize(60);
		changeText("00:00:00");
	}
	
	/**
	 * Stop recording without saving workout (For example if GPS sensor is deactivated)
	 */
	public void quitRecording() {
		recording = false;


		//Stop recording sensors
		recordSensorsThread.cleanUp();		

		SQLiteDatabase db = getSqlHelper().getReadableDatabase();
		db.delete(MySQLiteHelper.TABLE_CLASSIFICATION_DATA, null, null);		
		db.delete(MySQLiteHelper.TABLE_ACCELERATION_DATA, null, null);		
		db.delete(MySQLiteHelper.TABLE_GPS_DATA, null, null);	

		workoutFragment = new WorkoutFragment();
		getFragmentManager().beginTransaction().replace(R.id.container, workoutFragment).commit();
	}

	/**
	 * Executed after pressing the stop recording button. Stops recording sensors and calculates statistics.
	 */
	public void stopRecording(View view) {
		showProgress();

		findViewById(R.id.startButton).setVisibility(View.VISIBLE);
		findViewById(R.id.stopButton).setVisibility(View.GONE);

		recording = false;

		//Stop recording sensors
		recordSensorsThread.cleanUp();
		
		runOnUiThread(new Runnable() {
			public void run() {
				TextView textView = (TextView) findViewById(R.id.log);
				textView.setText("");
			}
		});

		//Export database
		new ExportDatabaseCSVTask(WorkoutActivity.this).execute();
		
		calculateStatistics();		
	}

	/**
	 * Evaluates recorded data, checks for correctness and sends aggregated data to server and adds them to statistics.
	 */
	public void calculateStatistics() {

		SQLiteDatabase db = getSqlHelper().getReadableDatabase();

		//Check classification
		Cursor res =  db.rawQuery( "SELECT AVG(" + MySQLiteHelper.CLASSIFICATION_COLUMN_RESULT + ") FROM " + MySQLiteHelper.TABLE_CLASSIFICATION_DATA, null );
		res.moveToFirst();
		double classificationAverage = res.getDouble(0);
		Log.d("WorkoutActivity", "ClassificationAverage: " + classificationAverage);
		
		if(classificationAverage < 0.5) {
			new AlertDialog.Builder(this)
				.setIcon(android.R.drawable.ic_dialog_alert)
				.setTitle(getResources().getText(R.string.error))
				.setMessage(getResources().getText(R.string.activity_recognition_rate_too_low) + " (" + Math.round(classificationAverage*100) + "%)")
				.setPositiveButton(getResources().getText(R.string.ok),null)
				.show();
			hideProgress();
			workoutFragment = new WorkoutFragment();
			getFragmentManager().beginTransaction().replace(R.id.container, workoutFragment).commit();
			workoutMode = WorkoutMode.RUNNING;
			return;
		}


		//Get start and end time
		res =  db.rawQuery( "SELECT MIN(" + MySQLiteHelper.ACCELERATION_COLUMN_TIMESTAMP + "), MAX(" + MySQLiteHelper.ACCELERATION_COLUMN_TIMESTAMP + ") FROM " + MySQLiteHelper.TABLE_ACCELERATION_DATA, null );
		res.moveToFirst();
		long startTime = res.getLong(0);
		long endTime = res.getLong(1);
		int totalSeconds = ((Long) ((endTime-startTime)/1000)).intValue();

		//Get speed average
		res =  db.rawQuery( "SELECT AVG(" + MySQLiteHelper.GPS_COLUMN_SPEED + ") FROM " + MySQLiteHelper.TABLE_GPS_DATA, null );
		res.moveToFirst();
		double speedMetersPerSecond = res.getDouble(0);
		double speed = Math.round(speedMetersPerSecond*3.6*100)/100;

		if(speed < 3.0) {
			new AlertDialog.Builder(this)
				.setIcon(android.R.drawable.ic_dialog_alert)
				.setTitle(getResources().getText(R.string.error))
				.setMessage(getResources().getText(R.string.average_speed_below_minimum) + " (3 " + getResources().getText(R.string.kmh) + ")")
				.setPositiveButton(getResources().getText(R.string.ok),null)
				.show();
			hideProgress();
			workoutFragment = new WorkoutFragment();
			getFragmentManager().beginTransaction().replace(R.id.container, workoutFragment).commit();			
			workoutMode = WorkoutMode.RUNNING;
			return;
		}
		
		//Calculate features
		int distance = (int) speed*totalSeconds*1000/3600;
		int caloriesBurned = 0;
		int credits = 0;
		//TODO difference between running and cycling
		
		if(workoutMode.equals(WorkoutMode.RUNNING)) {
			caloriesBurned = ((Long) Math.round(user.getBodyWeight()*(2.3+0.3*Math.pow(speed-2.5,1.65)*totalSeconds/3600))).intValue();
			credits = ((Long) Math.round(Math.pow(speed-2.5,1.65)*totalSeconds*1000/3600/10)).intValue();
		}else if(workoutMode.equals(WorkoutMode.CYCLING)) {
			caloriesBurned = ((Long) Math.round((speedMetersPerSecond*(3.509+(0.2581*Math.pow(speedMetersPerSecond, 2))+10.32*user.getBodyWeight()))*totalSeconds/4168)).intValue();
			credits = ((Long) Math.round(Math.pow(speed-2.5,1.65)*totalSeconds*1000/3600/10/5)).intValue();
		}


		//Create statistic object
		Statistic statistic = new Statistic();
		statistic.setUser(user);
		statistic.setStartTime(startTime);
		statistic.setTotalSeconds(totalSeconds);
		statistic.setDistance(distance);
		statistic.setSpeed(speed);
		statistic.setCaloriesBurned(caloriesBurned);
		statistic.setCredits(credits);
		statistic.setWorkoutMode(workoutMode);
		
		//Update total credits
		user.setTotalCredits(user.getTotalCredits()+credits);	

		server.submitStatistics(statistic);

		//Show statistics on screen
		StatisticsDetailFragment sdf = new StatisticsDetailFragment(statistic);
		getFragmentManager().beginTransaction().replace(R.id.container, sdf).commit();		
	}
	
	/**
	 * Showing statistic fragment after clicking "back" button at workout summary.
	 * @param view
	 */
	public void back(View view) {
		if(statisticsFragment == null) statisticsFragment = new StatisticsFragment(this, statistics);
		getFragmentManager().beginTransaction().replace(R.id.container, statisticsFragment).commit();		
	}
	
	/**
	 * Reads out changed settings and sends them to server
	 * @param view
	 */
	public void settingsSaveDo(View view) {
		String password = ((EditText) findViewById(R.id.tablePasswordValue)).getText().toString();
		String bodyWeight = ((EditText) findViewById(R.id.tableBodyWeightValue)).getText().toString();
		
		server.saveSettings(user.getUsername(), user.getPassword(), password, bodyWeight);		
	}
	
	/**
	 * Change active mode in workout fragment to running.
	 * @param view
	 */
	public void changeModeToRunning(View view) {
		workoutMode = WorkoutMode.RUNNING;
		((ImageView) findViewById(R.id.workoutModeRunningImage)).setImageResource(R.drawable.workout_running_active);
		((TextView) findViewById(R.id.workoutModeRunningText)).setTextColor(getResources().getColor(R.color.workoutModeTextActive));

		((ImageView) findViewById(R.id.workoutModeCyclingImage)).setImageResource(R.drawable.workout_cycling_inactive);
		((TextView) findViewById(R.id.workoutModeCyclingText)).setTextColor(getResources().getColor(R.color.workoutModeTextInactive));
	}
	
	/**
	 * Change active mode in workout fragment to cycling.
	 * @param view
	 */	
	public void changeModeToCycling(View view) {
		workoutMode = WorkoutMode.CYCLING;
		((ImageView) findViewById(R.id.workoutModeRunningImage)).setImageResource(R.drawable.workout_running_inactive);
		((TextView) findViewById(R.id.workoutModeRunningText)).setTextColor(getResources().getColor(R.color.workoutModeTextInactive));

		((ImageView) findViewById(R.id.workoutModeCyclingImage)).setImageResource(R.drawable.workout_cycling_active);
		((TextView) findViewById(R.id.workoutModeCyclingText)).setTextColor(getResources().getColor(R.color.workoutModeTextActive));
	}

	/**
	 * Changes text (elapsed time) in workout fragment.
	 * @param text
	 */
	public void changeText(final String text) {
		runOnUiThread(new Runnable() {
			@Override
			public void run() {
				TextView textView = (TextView) findViewById(R.id.time);
				if(textView != null)
					textView.setText(text);								
			}
		});
	}
	
	public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
		//Show statistics on screen
		StatisticsDetailFragment sdf = new StatisticsDetailFragment(statistics.get(position));
		getFragmentManager().beginTransaction().replace(R.id.container, sdf).commit();		
	}	

	public void addStatistic(Statistic statistic) {
		statistics.add(0, statistic);
	}

	public void showProgress() {
		progressDialog.show();
	}

	public void hideProgress() {
		progressDialog.hide();
	}

	public MySQLiteHelper getSqlHelper() {
		return sqlHelper;
	}

	public SQLiteDatabase getDatabase() {
		return database;
	}

	public boolean isRecording() {
		return recording;
	}
	
	public void setUser(UserProfile user) {
		this.user = user;
	}

	public WorkoutMode getWorkoutMode() {
		return workoutMode;
	}

	public long getStartRecordingTime() {
		return startRecordingTime;
	}	
}
