package com.example.paybyworkout.classifier;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import libsvm.svm;
import libsvm.svm_model;
import libsvm.svm_node;
import android.content.ContentValues;
import android.content.Context;
import android.os.Vibrator;
import android.util.Log;
import android.widget.TextView;

import com.example.paybyworkout.WorkoutActivity;
import com.example.paybyworkout.R;
import com.example.paybyworkout.classes.ClassificationFeatures;
import com.example.paybyworkout.classes.WorkoutMode;
import com.example.paybyworkout.database.MySQLiteHelper;

/**
 * Currently unused - Naive Bayes classification used instead!
 * Uses LibSVM algorithm for classification. Reads the classification features instance and classifies it by the given model. 
 * Afterwards the classification result gets stored in the database and the UI of the workout fragment gets updated.
 * @author Jan
 */
public class LibSVMClassifierThread extends Thread {

	private WorkoutActivity activity;
	private ClassificationFeatures classificationFeatures;
	private svm_model model;
	private double[][] scaling;
	private double prediction;
	private svm_node[] x;
	private Vibrator vibrator;

	public LibSVMClassifierThread(WorkoutActivity activity, ClassificationFeatures classificationFeatures) {
		this.activity = activity;
		this.classificationFeatures = classificationFeatures;
		vibrator = (Vibrator) activity.getSystemService(Context.VIBRATOR_SERVICE);

		try {

			//Get number of rows
			BufferedReader scalingParameterCount = null;
			BufferedReader scaling_parameters = null;

			if(activity.getWorkoutMode().equals(WorkoutMode.RUNNING)) {
				model = svm.svm_load_model(new BufferedReader(new InputStreamReader(activity.getResources().openRawResource(R.raw.train_running))));				
				scalingParameterCount = new BufferedReader(new InputStreamReader(activity.getResources().openRawResource(R.raw.scaling_running)));
				scaling_parameters = new BufferedReader(new InputStreamReader(activity.getResources().openRawResource(R.raw.scaling_running)));
			}else if(activity.getWorkoutMode().equals(WorkoutMode.CYCLING)) {
				model = svm.svm_load_model(new BufferedReader(new InputStreamReader(activity.getResources().openRawResource(R.raw.train_cycling))));
				scalingParameterCount = new BufferedReader(new InputStreamReader(activity.getResources().openRawResource(R.raw.scaling_cycling)));
				scaling_parameters = new BufferedReader(new InputStreamReader(activity.getResources().openRawResource(R.raw.scaling_cycling)));
			}

			//Read first line
			scalingParameterCount.readLine();
			scalingParameterCount.readLine();

			int parameterCount = 0;
			while (scalingParameterCount.readLine() != null) {
				parameterCount++;
			}

			//Fill scaling array;
			scaling = new double[parameterCount][2];
			String scaling_parameter;

			//Read first two lines
			scaling_parameters.readLine();
			scaling_parameters.readLine();

			int parameterNumber = 0;
			while ((scaling_parameter = scaling_parameters.readLine()) != null) {
				String[] line = scaling_parameter.replace(',', '.').split(" ");
				scaling[parameterNumber][0] = Double.parseDouble(line[1]);
				scaling[parameterNumber][1] = Double.parseDouble(line[2]);
				parameterNumber++;
			}

		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void run() {	

		x = new svm_node[32];
		x[0] = new svm_node();
		x[0].index = 0;
		if(classificationFeatures.getClassification().equals(WorkoutMode.OTHER)) {
			x[0].value = 0;			
		}else{
			x[0].value = 1;
		}

		x[1] = new svm_node();
		x[1].index = 1;
		x[1].value = classificationFeatures.getAccelerationAverageX();

		x[2] = new svm_node();
		x[2].index = 2;
		x[2].value = classificationFeatures.getAccelerationAverageY();

		x[3] = new svm_node();
		x[3].index = 3;
		x[3].value = classificationFeatures.getAccelerationAverageZ();

		x[4] = new svm_node();
		x[4].index = 4;
		x[4].value = classificationFeatures.getAccelerationXYcorrelation();

		x[5] = new svm_node();
		x[5].index = 5;
		x[5].value = classificationFeatures.getAccelerationXZcorrelation();

		x[6] = new svm_node();
		x[6].index = 6;
		x[6].value = classificationFeatures.getAccelerationYZcorrelation();

		x[7] = new svm_node();
		x[7].index = 7;
		x[7].value = classificationFeatures.getxVariance();

		x[8] = new svm_node();
		x[8].index = 8;
		x[8].value = classificationFeatures.getyVariance();

		x[9] = new svm_node();
		x[9].index = 9;
		x[9].value = classificationFeatures.getzVariance();

		x[10] = new svm_node();
		x[10].index = 10;
		x[10].value = classificationFeatures.getxMin();

		x[11] = new svm_node();
		x[11].index = 11;
		x[11].value = classificationFeatures.getxMax();

		x[12] = new svm_node();
		x[12].index = 12;
		x[12].value = classificationFeatures.getyMin();

		x[13] = new svm_node();
		x[13].index = 13;
		x[13].value = classificationFeatures.getyMax();

		x[14] = new svm_node();
		x[14].index = 14;
		x[14].value = classificationFeatures.getzMin();

		x[15] = new svm_node();
		x[15].index = 15;
		x[15].value = classificationFeatures.getzMax();

		x[16] = new svm_node();
		x[16].index = 16;
		x[16].value = classificationFeatures.getAccelerationAverageXFFT();

		x[17] = new svm_node();
		x[17].index = 17;
		x[17].value = classificationFeatures.getAccelerationAverageYFFT();

		x[18] = new svm_node();
		x[18].index = 18;
		x[18].value = classificationFeatures.getAccelerationAverageZFFT();		

		x[19] = new svm_node();
		x[19].index = 19;
		x[19].value = classificationFeatures.getAccelerationXYcorrelationFFT();

		x[20] = new svm_node();
		x[20].index = 20;
		x[20].value = classificationFeatures.getAccelerationXZcorrelationFFT();

		x[21] = new svm_node();
		x[21].index = 21;
		x[21].value = classificationFeatures.getAccelerationYZcorrelationFFT();

		x[22] = new svm_node();
		x[22].index = 22;
		x[22].value = classificationFeatures.getxVarianceFFT();

		x[23] = new svm_node();
		x[23].index = 23;
		x[23].value = classificationFeatures.getyVarianceFFT();

		x[24] = new svm_node();
		x[24].index = 24;
		x[24].value = classificationFeatures.getzVarianceFFT();

		x[25] = new svm_node();
		x[25].index = 25;
		x[25].value = classificationFeatures.getxMinFFT();

		x[26] = new svm_node();
		x[26].index = 26;
		x[26].value = classificationFeatures.getxMaxFFT();

		x[27] = new svm_node();
		x[27].index = 27;
		x[27].value = classificationFeatures.getyMinFFT();

		x[28] = new svm_node();
		x[28].index = 28;
		x[28].value = classificationFeatures.getyMaxFFT();

		x[29] = new svm_node();
		x[29].index = 29;
		x[29].value = classificationFeatures.getzMinFFT();

		x[30] = new svm_node();
		x[30].index = 30;
		x[30].value = classificationFeatures.getzMaxFFT();

		x[31] = new svm_node();
		x[31].index = 31;
		x[31].value = classificationFeatures.getAverageSpeed();		

		String scaledValuesString = "";
		//Scale x
		for(int i = 0; i < scaling.length; i++) {
			double unscaledValue = x[i+1].value;
			double scaledValue = -1 + 2*(unscaledValue-scaling[i][0])/(scaling[i][1]-scaling[i][0]);
			x[i+1].value = scaledValue;
			scaledValuesString += i + ": " + scaledValue + " ";
		}

		prediction = svm.svm_predict(model, x);

		//Insert into database
		ContentValues contentValues = new ContentValues();
		contentValues.put(MySQLiteHelper.CLASSIFICATION_COLUMN_TIMESTAMP, System.currentTimeMillis());
		contentValues.put(MySQLiteHelper.CLASSIFICATION_COLUMN_RESULT, String.valueOf(prediction));	
		activity.getDatabase().insert(MySQLiteHelper.TABLE_CLASSIFICATION_DATA, null, contentValues);

		Log.d("LibSVMClassifierThread", "Classification: " + prediction + " with values " + scaledValuesString);

		activity.runOnUiThread(new Runnable() {
			public void run() {
				TextView statusText = ((TextView)activity.findViewById(R.id.status));
				TextView timeText = ((TextView)activity.findViewById(R.id.time));

				if(prediction == 1) {
					timeText.setTextColor(activity.getResources().getColor(R.color.workoutRunning));
					statusText.setTextColor(activity.getResources().getColor(R.color.workoutRunning));
					if(activity.getWorkoutMode().equals(WorkoutMode.RUNNING)) {
						statusText.setText(activity.getResources().getString(R.string.running));
					}else if(activity.getWorkoutMode().equals(WorkoutMode.CYCLING)) {
						statusText.setText(activity.getResources().getString(R.string.cycling));					
					}
					vibrator.cancel();
						
					/*
						logText.setText("Classified as: RUNNING\n\n"
								+ "1: " + classificationFeatures.getAccelerationAverage() + "\n"
								+ "2: " + classificationFeatures.getAccelerationXYcorrelation() + "\n"
								+ "3: " + classificationFeatures.getAccelerationXZcorrelation() + "\n"
								+ "4: " + classificationFeatures.getAccelerationYZcorrelation() + "\n"
								+ "5: " + classificationFeatures.getxVariance() + "\n"
								+ "6: " + classificationFeatures.getyVariance() + "\n"
								+ "7: " + classificationFeatures.getzVariance() + "\n"
								+ "8: " + classificationFeatures.getxMin() + "\n"
								+ "9: " + classificationFeatures.getxMax() + "\n"
								+ "10: " + classificationFeatures.getyMin() + "\n"
								+ "11: " + classificationFeatures.getyMax() + "\n"
								+ "12: " + classificationFeatures.getzMin() + "\n"
								+ "13: " + classificationFeatures.getzMax() + "\n"
								+ "14: " + classificationFeatures.getAccelerationAverageFFT() + "\n"
								+ "15: " + classificationFeatures.getAccelerationXYcorrelationFFT() + "\n"
								+ "16: " + classificationFeatures.getAccelerationXZcorrelationFFT() + "\n"
								+ "17: " + classificationFeatures.getAccelerationYZcorrelationFFT() + "\n"
								+ "18: " + classificationFeatures.getxVarianceFFT() + "\n"
								+ "19: " + classificationFeatures.getyVarianceFFT() + "\n"
								+ "20: " + classificationFeatures.getzVarianceFFT() + "\n"
								+ "21: " + classificationFeatures.getxMinFFT() + "\n"
								+ "22: " + classificationFeatures.getxMaxFFT() + "\n"
								+ "23: " + classificationFeatures.getyMinFFT() + "\n"
								+ "24: " + classificationFeatures.getyMaxFFT() + "\n"
								+ "25: " + classificationFeatures.getzMinFFT() + "\n"
								+ "26: " + classificationFeatures.getzMaxFFT() + "\n"
								+ "27: " + classificationFeatures.getAverageSpeed()						
								);
					 */
				}else{
					timeText.setTextColor(activity.getResources().getColor(R.color.workoutOther));
					statusText.setTextColor(activity.getResources().getColor(R.color.workoutOther));
					if(activity.getWorkoutMode().equals(WorkoutMode.RUNNING)) {
						statusText.setText(activity.getResources().getString(R.string.not_running));
					}else if(activity.getWorkoutMode().equals(WorkoutMode.CYCLING)) {
						statusText.setText(activity.getResources().getString(R.string.not_cycling));					
					}

					if(activity.isRecording()) {
						long[] pattern = {0,500,3000};
						vibrator.vibrate(pattern, 0);
					}
					/*
						logText.setText("Classified as: OTHER\n\n"
								+ "1: " + classificationFeatures.getAccelerationAverage() + "\n"
								+ "2: " + classificationFeatures.getAccelerationXYcorrelation() + "\n"
								+ "3: " + classificationFeatures.getAccelerationXZcorrelation() + "\n"
								+ "4: " + classificationFeatures.getAccelerationYZcorrelation() + "\n"
								+ "5: " + classificationFeatures.getxVariance() + "\n"
								+ "6: " + classificationFeatures.getyVariance() + "\n"
								+ "7: " + classificationFeatures.getzVariance() + "\n"
								+ "8: " + classificationFeatures.getxMin() + "\n"
								+ "9: " + classificationFeatures.getxMax() + "\n"
								+ "10: " + classificationFeatures.getyMin() + "\n"
								+ "11: " + classificationFeatures.getyMax() + "\n"
								+ "12: " + classificationFeatures.getzMin() + "\n"
								+ "13: " + classificationFeatures.getzMax() + "\n"
								+ "14: " + classificationFeatures.getAccelerationAverageFFT() + "\n"
								+ "15: " + classificationFeatures.getAccelerationXYcorrelationFFT() + "\n"
								+ "16: " + classificationFeatures.getAccelerationXZcorrelationFFT() + "\n"
								+ "17: " + classificationFeatures.getAccelerationYZcorrelationFFT() + "\n"
								+ "18: " + classificationFeatures.getxVarianceFFT() + "\n"
								+ "19: " + classificationFeatures.getyVarianceFFT() + "\n"
								+ "20: " + classificationFeatures.getzVarianceFFT() + "\n"
								+ "21: " + classificationFeatures.getxMinFFT() + "\n"
								+ "22: " + classificationFeatures.getxMaxFFT() + "\n"
								+ "23: " + classificationFeatures.getyMinFFT() + "\n"
								+ "24: " + classificationFeatures.getyMaxFFT() + "\n"
								+ "25: " + classificationFeatures.getzMinFFT() + "\n"
								+ "26: " + classificationFeatures.getzMaxFFT() + "\n"
								+ "27: " + classificationFeatures.getAverageSpeed()								
								//							+ "1: " + x[1].value + "\n"
								//							+ "2: " + x[2].value + "\n"
								//							+ "3: " + x[3].value + "\n"
								//							+ "4: " + x[4].value + "\n"
								//							+ "5: " + x[5].value + "\n"
								//							+ "6: " + x[6].value + "\n"
								//							+ "7: " + x[7].value + "\n"
								//							+ "8: " + x[8].value + "\n"
								//							+ "9: " + x[9].value + "\n"
								//							+ "10: " + x[10].value + "\n"
								//							+ "11: " + x[11].value + "\n"
								//							+ "12: " + x[12].value + "\n"
								//							+ "13: " + x[13].value + "\n"
								//							+ "14: " + x[14].value					
								);
					 */
				}
			}
		});
	}
}
