package com.example.paybyworkout.database;

import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.Locale;

import android.app.ProgressDialog;
import android.database.Cursor;
import android.database.sqlite.SQLiteDatabase;
import android.os.AsyncTask;
import android.os.Environment;
import android.util.Log;

import com.example.paybyworkout.WorkoutActivity;
import com.opencsv.CSVWriter;

//new async task for file export to csv
/**
 * Exports all data in the database into .csv files to the local storage.
 * @author Jan
 */
public class ExportDatabaseCSVTask extends AsyncTask<String, String, Boolean> {
	
    private final ProgressDialog dialog;
    private WorkoutActivity activity;
    boolean memoryErr = false;	
	
	public ExportDatabaseCSVTask(WorkoutActivity activity) {
		dialog = new ProgressDialog(activity);
		this.activity = activity;
	}

    // to show Loading dialog box
    @Override
    protected void onPreExecute() {
        this.dialog.setMessage("Exporting database...");
        this.dialog.show();
    }

    // to write process 
    protected Boolean doInBackground(final String... args) {
        Log.d("ExportDB", "started"); // get the path of db

        boolean success = false;

        String currentDateString = new SimpleDateFormat("yyyy_MM_d_HH_mm_ss", Locale.GERMAN).format(new Date());

        File dbFile = activity.getDatabasePath(MySQLiteHelper.DB_NAME);
        Log.d("ExportDB", "Db path is: " + dbFile); // get the path of db
        File exportDir = new File(Environment.getExternalStorageDirectory() + File.separator + "PbW_Files", "");
        Log.d("ExportDB", "ExportDir is: " + exportDir); // get the path of db

        long freeBytesInternal = new File(activity.getApplicationContext().getFilesDir().getAbsoluteFile().toString()).getFreeSpace();           
        long megAvailable = freeBytesInternal / 1048576;

        if (megAvailable < 0.1) {               
            System.out.println("Please check"+megAvailable);
            memoryErr = true;               
        }else {             
            if (!exportDir.exists()) {
                exportDir.mkdirs();
            }   
            try {
                File file = new File(exportDir, currentDateString + "_" + MySQLiteHelper.TABLE_GPS_DATA + ".csv");
                file.createNewFile();
                CSVWriter csvWrite = new CSVWriter(new FileWriter(file),',',CSVWriter.NO_QUOTE_CHARACTER);
                csvWrite.writeNext(MySQLiteHelper.GPS_COLUMNS);

				SQLiteDatabase db = activity.getSqlHelper().getReadableDatabase();
				Cursor res =  db.rawQuery( "select * from " + MySQLiteHelper.TABLE_GPS_DATA, null );
				res.moveToFirst();
				while(res.isAfterLast() == false){
					String[] writeNext = {res.getString(0), res.getString(1), res.getString(2), res.getString(3), res.getString(4), res.getString(5)};
					csvWrite.writeNext(writeNext);
					res.moveToNext();
				}
                csvWrite.close();
				db.delete(MySQLiteHelper.TABLE_GPS_DATA, null, null);

                file = new File(exportDir, currentDateString + "_" + MySQLiteHelper.TABLE_ACCELERATION_DATA + ".csv");
                file.createNewFile();
                csvWrite = new CSVWriter(new FileWriter(file),',',CSVWriter.NO_QUOTE_CHARACTER);
                csvWrite.writeNext(MySQLiteHelper.ACCELERATION_COLUMNS);					
				res =  db.rawQuery( "select * from " + MySQLiteHelper.TABLE_ACCELERATION_DATA, null );
				res.moveToFirst();
				while(res.isAfterLast() == false){
					String[] writeNext = {res.getString(0), res.getString(1), res.getString(2), res.getString(3), res.getString(4)};
					csvWrite.writeNext(writeNext);
					res.moveToNext();
				}
				db.delete(MySQLiteHelper.TABLE_ACCELERATION_DATA, null, null);
                csvWrite.close();

                file = new File(exportDir, currentDateString + "_" + MySQLiteHelper.TABLE_CLASSIFICATION_DATA + ".csv");
                file.createNewFile();
                csvWrite = new CSVWriter(new FileWriter(file),',',CSVWriter.NO_QUOTE_CHARACTER);
                csvWrite.writeNext(MySQLiteHelper.CLASSIFICATION_COLUMNS);
                res =  db.rawQuery( "select * from " + MySQLiteHelper.TABLE_CLASSIFICATION_DATA, null );
				res.moveToFirst();
				while(res.isAfterLast() == false){
					String[] writeNext = {res.getString(0), res.getString(1), res.getString(2)};
					csvWrite.writeNext(writeNext);
					res.moveToNext();
				}
				db.delete(MySQLiteHelper.TABLE_CLASSIFICATION_DATA, null, null);
                csvWrite.close();
                
				success = true;

            } catch (IOException e) {
                Log.e("SearchResultActivity", e.getMessage(), e);
                return success;
            }
        }
        return success;
    }

    // close dialog and give msg
    protected void onPostExecute(Boolean success) {
        if (this.dialog.isShowing()) {
            this.dialog.dismiss();
        }
        if (success) {
            Log.d("ExportDB","success");
        } else {                
            if (memoryErr==true) {
	            Log.d("ExportDB","memoryError");
            } else {
	            Log.d("ExportDB","otherError");
            }
        }
    }
}	