package com.example.paybyworkout.database;

import android.content.Context;
import android.database.sqlite.SQLiteDatabase;
import android.database.sqlite.SQLiteOpenHelper;
import android.util.Log;

/**
 * Responsible for the naming and access to the database
 * @author Jan
 */
public class MySQLiteHelper extends SQLiteOpenHelper {
	public static final String DB_NAME = "pbw_data.db";
	public static final int DB_VERSION = 1;

	//CLASSIFICATION Table
	public static final String TABLE_CLASSIFICATION_DATA = "classification";
	public static final String CLASSIFICATION_COLUMN_ID = "id";
	public static final String CLASSIFICATION_COLUMN_TIMESTAMP = "timestamp";
	public static final String CLASSIFICATION_COLUMN_RESULT = "x";
	public static final String[] CLASSIFICATION_COLUMNS = {CLASSIFICATION_COLUMN_ID, CLASSIFICATION_COLUMN_TIMESTAMP, CLASSIFICATION_COLUMN_RESULT};
	
	//GPS Table
	public static final String TABLE_GPS_DATA = "gps";
	public static final String GPS_COLUMN_ID = "id";
	public static final String GPS_COLUMN_TIMESTAMP = "timestamp";
	public static final String GPS_COLUMN_LAT = "lat";
	public static final String GPS_COLUMN_LON = "lon";
	public static final String GPS_COLUMN_ACCURACY = "accuracy";
	public static final String GPS_COLUMN_SPEED = "speed";
	public static final String[] GPS_COLUMNS = {GPS_COLUMN_ID, GPS_COLUMN_TIMESTAMP, GPS_COLUMN_LAT, GPS_COLUMN_LON, GPS_COLUMN_ACCURACY, GPS_COLUMN_SPEED};


	//ACCELERATION Table
	public static final String TABLE_ACCELERATION_DATA = "acceleration";
	public static final String ACCELERATION_COLUMN_ID = "id";
	public static final String ACCELERATION_COLUMN_TIMESTAMP = "timestamp";
	public static final String ACCELERATION_COLUMN_X = "x";
	public static final String ACCELERATION_COLUMN_Y = "y";
	public static final String ACCELERATION_COLUMN_Z = "z";
	public static final String[] ACCELERATION_COLUMNS = {ACCELERATION_COLUMN_ID, ACCELERATION_COLUMN_TIMESTAMP, ACCELERATION_COLUMN_X, ACCELERATION_COLUMN_Y, ACCELERATION_COLUMN_Z};

//	//GYROSCOPE Table
//	public static final String TABLE_GYROSCOPE_DATA = "gyroscope";
//	public static final String GYROSCOPE_COLUMN_ID = "id";
//	public static final String GYROSCOPE_COLUMN_TIMESTAMP = "timestamp";
//	public static final String GYROSCOPE_COLUMN_X = "x";
//	public static final String GYROSCOPE_COLUMN_Y = "y";
//	public static final String GYROSCOPE_COLUMN_Z = "z";
//	public static final String[] GYROSCOPE_COLUMNS = {GYROSCOPE_COLUMN_ID, GYROSCOPE_COLUMN_TIMESTAMP, GYROSCOPE_COLUMN_X, GYROSCOPE_COLUMN_Y, GYROSCOPE_COLUMN_Z};
	
	public static final String CREATE_TABLE_CLASSIFICATION_DATA = "CREATE TABLE "
		      + TABLE_CLASSIFICATION_DATA + "("
		      + CLASSIFICATION_COLUMN_ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
		      + CLASSIFICATION_COLUMN_TIMESTAMP + " TIMESTAMP,"
		      + CLASSIFICATION_COLUMN_RESULT + " TEXT"
		      + "); ";
	
	public static final String CREATE_TABLE_GPS_DATA = "CREATE TABLE "
		      + TABLE_GPS_DATA + "("
		      + GPS_COLUMN_ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
		      + GPS_COLUMN_TIMESTAMP + " TIMESTAMP,"
		      + GPS_COLUMN_LAT + " TEXT,"
		      + GPS_COLUMN_LON + " TEXT,"
		      + GPS_COLUMN_ACCURACY + " TEXT,"
		      + GPS_COLUMN_SPEED + " TEXT"
		      + "); ";
	
	public static final String CREATE_TABLE_ACCELERATION_DATA = "CREATE TABLE "
		      + TABLE_ACCELERATION_DATA + "("
		      + ACCELERATION_COLUMN_ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
		      + ACCELERATION_COLUMN_TIMESTAMP + " TIMESTAMP,"
		      + ACCELERATION_COLUMN_X + " TEXT,"
		      + ACCELERATION_COLUMN_Y + " TEXT,"
		      + ACCELERATION_COLUMN_Z + " TEXT"
		      + ");";
	
//	public static final String CREATE_TABLE_GYROSCOPE_DATA = "CREATE TABLE "
//		      + TABLE_GYROSCOPE_DATA + "("
//		      + GYROSCOPE_COLUMN_ID + " INTEGER PRIMARY KEY AUTOINCREMENT, "
//		      + GYROSCOPE_COLUMN_TIMESTAMP + " TIMESTAMP,"
//		      + GYROSCOPE_COLUMN_X + " TEXT,"
//		      + GYROSCOPE_COLUMN_Y + " TEXT,"
//		      + GYROSCOPE_COLUMN_Z + " TEXT"
//		      + ");";
	
	public MySQLiteHelper(Context context) {
		super(context, DB_NAME, null, DB_VERSION);
	}

	@Override
	public void onCreate(SQLiteDatabase db) {
    	db.execSQL("DROP TABLE IF EXISTS " + TABLE_CLASSIFICATION_DATA);
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_GPS_DATA);
        db.execSQL("DROP TABLE IF EXISTS " + TABLE_ACCELERATION_DATA);
//        db.execSQL("DROP TABLE IF EXISTS " + TABLE_GYROSCOPE_DATA);
		db.execSQL(CREATE_TABLE_CLASSIFICATION_DATA);		
	    db.execSQL(CREATE_TABLE_GPS_DATA);		
	    db.execSQL(CREATE_TABLE_ACCELERATION_DATA);		
//	    db.execSQL(CREATE_TABLE_GYROSCOPE_DATA);		
	}

	@Override
	public void onUpgrade(SQLiteDatabase db, int oldVersion, int newVersion) {
	    Log.d(MySQLiteHelper.class.getName(),
	            "Upgrading database from version " + oldVersion + " to "
	                + newVersion + ", which will destroy all old data");
        	db.execSQL("DROP TABLE IF EXISTS " + TABLE_CLASSIFICATION_DATA);
	        db.execSQL("DROP TABLE IF EXISTS " + TABLE_GPS_DATA);
	        db.execSQL("DROP TABLE IF EXISTS " + TABLE_ACCELERATION_DATA);
//	        db.execSQL("DROP TABLE IF EXISTS " + TABLE_GYROSCOPE_DATA);
	        onCreate(db);	
	}

}
