package com.example.paybyworkout.record;

import java.text.SimpleDateFormat;
import java.util.Calendar;
import java.util.Locale;

import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.hardware.Sensor;
import android.hardware.SensorManager;
import android.location.LocationManager;
import android.provider.Settings;

import com.example.paybyworkout.R;
import com.example.paybyworkout.WorkoutActivity;

/**
 * Initializes both GPS and accelerometer recording and updates time of the UI every second.
 * @author Jan
 */
public class RecordSensorsThread extends Thread {

	public static final long UPDATE_INTERVAL = 100;
	public static final long GPS_UPDATE_INTERVAL = 1000;

	private WorkoutActivity activity;
	private SensorManager sensorManager;
	private AccelerometerListener accelerometerListener;
//	private GyroscopeListener gyroscopeListener;
	private LocationManager locationManager;
	private GPSListener gpsListener;
	private long calendarStart;

	public RecordSensorsThread(WorkoutActivity activity) {
		this.activity = activity;
	}

	public void cleanUp() {
		sensorManager.unregisterListener(accelerometerListener);
//		sensorManager.unregisterListener(gyroscopeListener);
		if(gpsListener != null) {
			locationManager.removeUpdates(gpsListener);
		}
	}

	public void run() {	
		Calendar c = Calendar.getInstance();
		c.set(Calendar.HOUR_OF_DAY, 0);
		c.set(Calendar.MINUTE, 0);
		c.set(Calendar.SECOND, 0);
		c.set(Calendar.MILLISECOND, 0);
		calendarStart = c.getTimeInMillis();
		
		//		//Initialize Gyroscope
//		sensorManager = (SensorManager) activity.getSystemService(Context.SENSOR_SERVICE);
//		Sensor senGyroscope = sensorManager.getDefaultSensor(Sensor.TYPE_GYROSCOPE);
//		gyroscopeListener = new GyroscopeListener(activity.getDatabase());
//		sensorManager.registerListener(gyroscopeListener, senGyroscope, ((Long) (UPDATE_INTERVAL*1000)).intValue());					

		//Initialize Accelerometer
		sensorManager = (SensorManager) activity.getSystemService(Context.SENSOR_SERVICE);
		Sensor senAccelerometer = sensorManager.getDefaultSensor(Sensor.TYPE_ACCELEROMETER);
		accelerometerListener = new AccelerometerListener(activity.getDatabase());
		sensorManager.registerListener(accelerometerListener, senAccelerometer, ((Long) (UPDATE_INTERVAL*1000)).intValue());			

		//Initialize GPS
		activity.runOnUiThread(new Runnable() {
			public void run() {			
				locationManager = (LocationManager) activity.getSystemService(WorkoutActivity.LOCATION_SERVICE);
				boolean enabled = locationManager.isProviderEnabled(LocationManager.GPS_PROVIDER);

				// check if enabled and if not send user to the GSP settings
				// Better solution would be to display a dialog and suggesting to 
				// go to the settings
				if (!enabled) {
					activity.quitRecording();
					new AlertDialog.Builder(activity)
					.setIcon(android.R.drawable.ic_dialog_alert)
					.setTitle(R.string.gps_error)
					.setMessage(R.string.error_location_service_not_activated)
					.setPositiveButton(R.string.activate_gps_here, new DialogInterface.OnClickListener()
					{
						@Override
						public void onClick(DialogInterface dialog, int which) {
							Intent intent = new Intent(Settings.ACTION_LOCATION_SOURCE_SETTINGS);
							activity.startActivity(intent);				
						}

					})
					.setNegativeButton(R.string.cancel, null)
					.show();
				}
				gpsListener = new GPSListener(activity.getDatabase());
				locationManager.requestLocationUpdates(LocationManager.GPS_PROVIDER, 0, 0, gpsListener);
			}
		});

		while(activity.isRecording()) {
			try {					
				//Update time in UI
				long passedSeconds = System.currentTimeMillis() - activity.getStartRecordingTime();
				c.setTimeInMillis(calendarStart + passedSeconds);
				SimpleDateFormat df = new SimpleDateFormat("HH:mm:ss", Locale.GERMAN);
				activity.changeText(df.format(c.getTime()));			    	

				Thread.sleep(1000);
			} catch (InterruptedException e) {
				return;
			}
		}
	}
}