package com.example.paybyworkout.server;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.List;

import org.apache.http.HttpResponse;
import org.apache.http.NameValuePair;
import org.apache.http.client.HttpClient;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.DefaultHttpClient;
import org.apache.http.message.BasicNameValuePair;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import android.app.Activity;
import android.content.Intent;
import android.os.AsyncTask;
import android.util.Log;
import android.widget.Toast;

import com.example.paybyworkout.LoginActivity;
import com.example.paybyworkout.R;
import com.example.paybyworkout.WorkoutActivity;
import com.example.paybyworkout.classes.Statistic;
import com.example.paybyworkout.classes.UserProfile;
import com.example.paybyworkout.classes.WorkoutMode;

/**
 * Communicates with the specified server
 * @author Jan
 */
public class Server {
	
	private final String serverURL = "http://www.annoart.at/paybyworkout/";
	private HttpClient httpClient = new DefaultHttpClient();
	private Activity activity;
	
	public Server(Activity activity) {
		this.activity = activity;
	}

	/**
	 * Starts a new task for checking credentials on server
	 * @param username
	 * @param password
	 */
	public void checkLogin(String username, String password) {	
		Log.d("Server", "CheckLogin with username " + username + " and password " + password);
		((LoginActivity) activity).showProgress();
		new CheckLoginTask().execute(username, password);
	}

	/**
	 * AsyncTask, which sends credentials to server. Receives user profile and statistics belonging to this user after successful login and opens the workout activity afterwards.
	 * @author Jan
	 */
	private class CheckLoginTask extends AsyncTask<String, String, String> {

		@Override
		protected String doInBackground(String... params) {
			
			//Connect to server with request and get user profile back	
			HttpPost postRequest = new HttpPost(serverURL + "request/check_login");
			HttpResponse response = null;
			BufferedReader bufferedReader = null;

			List<NameValuePair> postParams = new ArrayList<NameValuePair>();
			postParams.add(new BasicNameValuePair("username", params[0]));
			postParams.add(new BasicNameValuePair("password", params[1]));

			try {
				postRequest.setEntity(new UrlEncodedFormEntity(postParams));
				response = httpClient.execute(postRequest);
				bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
				
				StringBuffer stringBuffer = new StringBuffer("");
				String line = "";					
				
				while ((line = bufferedReader.readLine()) != null) {
					stringBuffer.append(line);
				}
				bufferedReader.close();					
				
				return stringBuffer.toString();
				
			} catch (IllegalStateException e) {
				e.printStackTrace();
	        	return "error";
			} catch (IOException e) {
				//No Internet Connection
	        	return "error";
			}	

		}

		protected void onPostExecute(String result) {
			super.onPostExecute(result);

			Log.d("Server","LoginResult: " + result);
			((LoginActivity) activity).hideProgress();
			if(result.equals("error")) {				
				Toast.makeText(activity.getApplicationContext(), R.string.wrong_credentials, Toast.LENGTH_LONG).show();
			}else{
				//Successfully logged in
				try {
					JSONObject jsonObject = new JSONObject(result);
					
					//Get user details
					UserProfile user = new UserProfile();
					JSONObject userJson = jsonObject.getJSONObject("user");
					user.setUsername(userJson.getString("username"));
					user.setPassword(userJson.getString("password"));
					user.setBodyWeight(userJson.getInt("bodyweight"));
					user.setTotalCredits(userJson.getInt("total_credits"));
					
					//Get statistics
					ArrayList<Statistic> statistics = new ArrayList<Statistic>();
					JSONArray statisticsJSON = jsonObject.getJSONArray("statistics");	
					for(int i = 0; i < statisticsJSON.length(); i++) {
						JSONObject statisticWrapperJSON = statisticsJSON.getJSONObject(i);
						JSONObject statisticJSON = statisticWrapperJSON.getJSONObject("Statistic");

						Statistic statistic = new Statistic();
						statistic.setUser(user);
						statistic.setStartTime(statisticJSON.getLong("start_time"));
						statistic.setTotalSeconds(statisticJSON.getInt("total_seconds"));
						statistic.setDistance(statisticJSON.getInt("distance"));
						statistic.setSpeed(statisticJSON.getDouble("speed"));
						statistic.setCaloriesBurned(statisticJSON.getInt("calories_burned"));
						statistic.setCredits(statisticJSON.getInt("credits"));
						int workoutMode = statisticJSON.getInt("workout_mode");
						if(workoutMode == 1) statistic.setWorkoutMode(WorkoutMode.RUNNING);
						if(workoutMode == 2) statistic.setWorkoutMode(WorkoutMode.CYCLING);
						
						statistics.add(statistic);
					}
					
					//Opening workout activity
					Intent intent = new Intent(activity, WorkoutActivity.class);		
					intent.putExtra("user", user);
					intent.putExtra("statistics", statistics);
					activity.startActivity(intent);
					activity.finish();	
				} catch (JSONException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					Toast.makeText(activity.getApplicationContext(), R.string.wrong_credentials, Toast.LENGTH_LONG).show();					
				}
			}
		}
	}	
	
	/**
	 * Sends data of new user to server and creates new account
	 * @param username
	 * @param password
	 * @param bodyWeight
	 */
	public void registerUser(String username, String password, String bodyWeight) {
		((LoginActivity) activity).showProgress();
		new RegisterUserTask().execute(username, password, bodyWeight);
	}
	

	/**
	 * AsyncTask, which sends user data to server and opens workout activity afterwards
	 * @author Jan
	 */
	private class RegisterUserTask extends AsyncTask<String, String, String> {

		@Override
		protected String doInBackground(String... params) {
			
			//Connect to server with request and get user profile back			
			HttpPost postRequest = new HttpPost(serverURL + "request/register_user");
			HttpResponse response = null;
			BufferedReader bufferedReader = null;

			List<NameValuePair> postParams = new ArrayList<NameValuePair>();
			postParams.add(new BasicNameValuePair("username", params[0]));
			postParams.add(new BasicNameValuePair("password", params[1]));
			postParams.add(new BasicNameValuePair("bodyweight", params[2]));

			try {
				postRequest.setEntity(new UrlEncodedFormEntity(postParams));
				response = httpClient.execute(postRequest);
				bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
				
				StringBuffer stringBuffer = new StringBuffer("");
				String line = "";					
				
				while ((line = bufferedReader.readLine()) != null) {
					stringBuffer.append(line);
				}
				bufferedReader.close();					
				
				return stringBuffer.toString();
				
			} catch (IllegalStateException e) {
				e.printStackTrace();
	        	return "error";
			} catch (IOException e) {
				//No Internet Connection
	        	return "error";
			}	

		}

		protected void onPostExecute(String result) {
			super.onPostExecute(result);

			((LoginActivity) activity).hideProgress();
			if(result.equals("error")) {				
				Toast.makeText(activity.getApplicationContext(), R.string.username_exists, Toast.LENGTH_LONG).show();
			}else{
				//Successfully logged in
				try {
					JSONObject jsonObject = new JSONObject(result);
					UserProfile user = new UserProfile();
					user.setUsername(jsonObject.getString("username"));
					user.setPassword(jsonObject.getString("password"));
					user.setBodyWeight(Integer.valueOf(jsonObject.getString("bodyweight")));

					//Create empty statistics list
					ArrayList<Statistic> statistics = new ArrayList<Statistic>();

					//Opening workout activity
					Intent intent = new Intent(activity, WorkoutActivity.class);		
					intent.putExtra("user", user);
					intent.putExtra("statistics", statistics);
					activity.startActivity(intent);
					activity.finish();	
				} catch (JSONException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					Toast.makeText(activity.getApplicationContext(), R.string.username_exists, Toast.LENGTH_LONG).show();					
				}
			}
		}
	}		

	
	/**
	 * Submit statistic data to server
	 * @param statistic
	 */
	public void submitStatistics(Statistic statistic) {
		String username = statistic.getUser().getUsername();
		String password = statistic.getUser().getPassword();
		String startTime = String.valueOf(statistic.getStartTime());
		String totalSeconds = String.valueOf(statistic.getTotalSeconds());
		String distance = String.valueOf(statistic.getDistance());
		String speed = String.valueOf(statistic.getSpeed());
		String caloriesBurned = String.valueOf(statistic.getCaloriesBurned());
		String credits = String.valueOf(statistic.getCredits());
		String workoutMode = "";
		if(statistic.getWorkoutMode().equals(WorkoutMode.RUNNING)) workoutMode = "1";
		if(statistic.getWorkoutMode().equals(WorkoutMode.CYCLING)) workoutMode = "2";
		
		
		new SubmitStatisticsTask(statistic).execute(username, password, startTime, totalSeconds, distance, speed, caloriesBurned, credits, workoutMode);
	}
	

	/**
	 * AsyncTask, which stores statistics data on server
	 * @author Jan
	 */
	private class SubmitStatisticsTask extends AsyncTask<String, String, String> {

		private Statistic statistic;
		
		public SubmitStatisticsTask(Statistic statistic) {
			this.statistic = statistic;
		}

		@Override
		protected String doInBackground(String... params) {
			
			//Connect to server with request and get user profile back			
			HttpPost postRequest = new HttpPost(serverURL + "request/submit_statistics");
			HttpResponse response = null;
			BufferedReader bufferedReader = null;

			List<NameValuePair> postParams = new ArrayList<NameValuePair>();
			postParams.add(new BasicNameValuePair("username", params[0]));
			postParams.add(new BasicNameValuePair("password", params[1]));
			postParams.add(new BasicNameValuePair("start_time", params[2]));
			postParams.add(new BasicNameValuePair("total_seconds", params[3]));
			postParams.add(new BasicNameValuePair("distance", params[4]));
			postParams.add(new BasicNameValuePair("speed", params[5]));
			postParams.add(new BasicNameValuePair("calories_burned", params[6]));
			postParams.add(new BasicNameValuePair("credits", params[7]));
			postParams.add(new BasicNameValuePair("workout_mode", params[8]));
			
			try {
				postRequest.setEntity(new UrlEncodedFormEntity(postParams));
				response = httpClient.execute(postRequest);
				bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
				
				StringBuffer stringBuffer = new StringBuffer("");
				String line = "";					
				
				while ((line = bufferedReader.readLine()) != null) {
					stringBuffer.append(line);
				}
				bufferedReader.close();					
				
				return stringBuffer.toString();
				
			} catch (IllegalStateException e) {
				e.printStackTrace();
	        	return "error";
			} catch (IOException e) {
				//No Internet Connection
	        	return "error";
			}	

		}

		protected void onPostExecute(String result) {
			super.onPostExecute(result);
			
			Log.d("Server","SubmitStatistics response: " + result);
			((WorkoutActivity) activity).hideProgress();
			if(result.equals("error")) {				
				Toast.makeText(activity.getApplicationContext(), R.string.server_not_available, Toast.LENGTH_LONG).show();
			}else{
				//Successfully logged in
				((WorkoutActivity) activity).addStatistic(statistic);
			}
		}
	}
	
	
	/**
	 * Change user settings on server
	 * @param username
	 * @param oldPassword
	 * @param password
	 * @param bodyWeight
	 */
	public void saveSettings(String username, String oldPassword, String password, String bodyWeight) {
		((WorkoutActivity) activity).showProgress();
		new SaveSettingsTask().execute(username, oldPassword, password, bodyWeight);
	}	

	/**
	 * AsyncTask, which saves user settings on server
	 * @author Jan
	 */
	private class SaveSettingsTask extends AsyncTask<String, String, String> {

		@Override
		protected String doInBackground(String... params) {
			
			//Connect to server with request and get user profile back			
			HttpPost postRequest = new HttpPost(serverURL + "request/save_settings");
			HttpResponse response = null;
			BufferedReader bufferedReader = null;

			List<NameValuePair> postParams = new ArrayList<NameValuePair>();
			postParams.add(new BasicNameValuePair("username", params[0]));
			postParams.add(new BasicNameValuePair("old_password", params[1]));
			postParams.add(new BasicNameValuePair("password", params[2]));
			postParams.add(new BasicNameValuePair("bodyweight", params[3]));

			try {
				postRequest.setEntity(new UrlEncodedFormEntity(postParams));
				response = httpClient.execute(postRequest);
				bufferedReader = new BufferedReader(new InputStreamReader(response.getEntity().getContent()));
				
				StringBuffer stringBuffer = new StringBuffer("");
				String line = "";					
				
				while ((line = bufferedReader.readLine()) != null) {
					stringBuffer.append(line);
				}
				bufferedReader.close();					
				
				return stringBuffer.toString();
				
			} catch (IllegalStateException e) {
				e.printStackTrace();
	        	return "error";
			} catch (IOException e) {
				//No Internet Connection
	        	return "error";
			}	

		}

		protected void onPostExecute(String result) {
			super.onPostExecute(result);

			((WorkoutActivity) activity).hideProgress();
			if(result.equals("error")) {				
				Toast.makeText(activity.getApplicationContext(), R.string.error_saving_settings, Toast.LENGTH_LONG).show();
			}else{
				//Successfully logged in
				try {
					JSONObject userJson = new JSONObject(result);
					
					//Get user details
					UserProfile user = new UserProfile();
					user.setUsername(userJson.getString("username"));
					user.setPassword(userJson.getString("password"));
					user.setBodyWeight(userJson.getInt("bodyweight"));
					user.setTotalCredits(userJson.getInt("total_credits"));
					
					((WorkoutActivity) activity).setUser(user);
					
					Toast.makeText(activity.getApplicationContext(), R.string.saving_successful, Toast.LENGTH_LONG).show();										
				} catch (JSONException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
					Toast.makeText(activity.getApplicationContext(), R.string.error_saving_settings, Toast.LENGTH_LONG).show();					
				}
			}
		}
	}		
}
